@@ -1,8 +1,8 @@
 #' @title Obtain dataset information from the Neotoma Paleoecological Database or an existing object.
 #' @description A function to access the Neotoma API and return datasets corresponding to the parameters defined by the user.
 #'
-#' @importFrom RCurl getForm
-#' @importFrom RJSONIO fromJSON
+#' @importFrom jsonlite fromJSON
+#' @importFrom httr content GET
 #' @param x An optional value, either a \code{numeric} site ID or object of class \code{download}, \code{download_list} or \code{site}.
 #' @param datasettype A character string corresponding to one of the allowed dataset types in the Neotoma Database.  Allowed types include: \code{"geochronologic"}, \code{"loss-on-ignition"}, \code{"pollen"}, \code{"plant macrofossils"}, \code{"vertebrate fauna"}, \code{"mollusks"}, and \code{"pollen surface sample"}.
 #' @param piid Numeric value for the Principle Investigator's ID number.
@@ -54,15 +54,15 @@
 #' @keywords IO connection
 #' @export
 #'
-get_dataset <- function(x, datasettype, piid, altmin, altmax, loc, gpid, taxonids, taxonname, ageold, ageyoung, ageof, subdate){
+get_dataset <- function(x, datasettype, piid, altmin, altmax, loc, gpid, taxonids, taxonname, ageold, ageyoung, ageof, subdate) {
   UseMethod('get_dataset')
 }
 
 #' @title Obtain dataset information from the Neotoma Paleoecological Database or an existing object.
 #' @description A function to access the Neotoma API and return datasets corresponding to the parameters defined by the user.
 #'
-#' @importFrom RCurl getForm
-#' @importFrom RJSONIO fromJSON
+#' @importFrom jsonlite fromJSON
+#' @importFrom httr GET content
 #' @param x A numeric value corresponding to the site ID.
 #' @param datasettype A character string corresponding to one of the allowed dataset types in the Neotoma Database.  Allowed types include: \code{"geochronologic"}, \code{"loss-on-ignition"}, \code{"pollen"}, \code{"plant macrofossils"}, \code{"vertebrate fauna"}, \code{"mollusks"}, and \code{"pollen surface sample"}.
 #' @param piid Numeric value for the Principle Investigator's ID number.
@@ -77,7 +77,7 @@ get_dataset <- function(x, datasettype, piid, altmin, altmax, loc, gpid, taxonid
 #' @param ageof If a taxon ID or taxon name is defined this parameter must be set to \code{"taxon"}, otherwise it may refer to \code{"sample"}, in which case the age bounds are for any samples within datasets or \code{"dataset"} if you want only datasets that are within the bounds of ageold and ageyoung.
 #' @param subdate Date of dataset submission, either YYYY-MM-DD or MM-DD-YYYY.
 #' @export
-get_dataset.default <- function(x, datasettype, piid, altmin, altmax, loc, gpid, taxonids, taxonname, ageold, ageyoung, ageof, subdate){
+get_dataset.default <- function(x, datasettype, piid, altmin, altmax, loc, gpid, taxonids, taxonname, ageold, ageyoung, ageof, subdate) {
   # The issue here is that these objects
   # have multiple tables of multiple lengths.
 
@@ -87,35 +87,47 @@ get_dataset.default <- function(x, datasettype, piid, altmin, altmax, loc, gpid,
   cl[[1]] <- NULL
   cl <- lapply(cl, eval, envir = parent.frame())
   
-  if('x' %in% names(cl)){
+  if ('x' %in% names(cl)) {
     names(cl)[which(names(cl) == 'x')] <- 'siteid'
   }
 
   #  Pass the parameters to param_check to make sure everything is kosher.
   error_test <- param_check(cl)
-  if(error_test[[2]]$flag == 1){
+  if (error_test[[2]]$flag == 1) {
     stop(paste0(unlist(error_test[[2]]$message), collapse='\n  '))
   } else {
     cl <- error_test[[1]]
   }
 
-  neotoma.form <- getForm(base.uri, .params = cl, binary = FALSE,
-                          .encoding = 'utf-16')
-
-  aa <- try(fromJSON(neotoma.form, nullValue = NA))
-
-  if (aa[[1]] == 0){
+  neotoma_content <- httr::content(httr::GET(base.uri, query = cl), as = "text")
+  if (identical(neotoma_content, "")) stop("")
+  aa <- jsonlite::fromJSON(neotoma_content, simplifyVector = FALSE)
+  
+  if (aa[[1]] == 0) {
     stop(paste('Server returned an error message:\n', aa[[2]]), call. = FALSE)
   }
-  if (aa[[1]] == 1){
+  if (aa[[1]] == 1) {
     output <- aa[[2]]
     
-    if(length(output) == 0){
+    rep_NULL <- function(x) { 
+      if (is.null(x)) {NA}
+      else{
+        if (class(x) == 'list') {
+          lapply(x, rep_NULL)
+        } else {
+          return(x)
+        }
+      }
+    }
+    
+    output <- lapply(output, function(x)rep_NULL(x))
+    
+    if (length(output) == 0) {
       warning('The criteria used returned 0 sample sites. Returning NULL.')
       return(NULL)
     }
     
-    if (length(aa[[2]]) > 1){
+    if (length(aa[[2]]) > 1) {
       message(paste('The API call was successful, you have returned ',
                     length(output), ' records.\n', sep = ''))
     } else {
@@ -126,7 +138,7 @@ get_dataset.default <- function(x, datasettype, piid, altmin, altmax, loc, gpid,
 
 
   if (inherits(output, "try-error")) {
-      new.output <- neotoma.form
+      new.output <- output
   } else {
       new.output <- lapply(output, function(x) {
           new.output <- list()
@@ -145,21 +157,29 @@ get_dataset.default <- function(x, datasettype, piid, altmin, altmax, loc, gpid,
 
           class(new.output$site.data) <- c('site', 'data.frame')
 
-          if('CollType' %in% names(x)){x$CollUnitType <- x$CollType} # This is a fix for a very specific issue we were having.
+          if ('CollType' %in% names(x)) {x$CollUnitType <- x$CollType} # This is a fix for a very specific issue we were having.
           
-          new.output$dataset.meta <- data.frame(dataset.id = x$DatasetID,
-                                                dataset.name = x$DatasetName,
-                                                collection.type = x$CollUnitType,
-                                                collection.handle = x$CollUnitHandle,
-                                                dataset.type = x$DatasetType,
+          new.output$dataset.meta <- data.frame(dataset.id = ifelse(class(x$DatasetID) == 'logical',
+                                                                    NA, x$DatasetID),
+                                                dataset.name = ifelse(class(x$DatasetName) == 'logical',
+                                                                      NA, x$DatasetName),
+                                                collection.type = ifelse(class(x$CollUnitType) == 'logical',
+                                                                         NA, x$CollUnitType),
+                                                collection.handle = ifelse(class(x$CollUnitHandle) == 'logical',
+                                                                           NA, x$CollUnitHandle),
+                                                dataset.type = ifelse(class(x$DatasetType) == 'logical',
+                                                                      NA, x$DatasetType),
                                                 stringsAsFactors = FALSE)
-
-          new.output$pi.data <- do.call(rbind.data.frame, x$DatasetPIs)
-          rownames(new.output$pi.data) <- NULL
+          if (class(x$DatasetPIs) == 'logical') { 
+            new.output$pi.data <- NA
+          } else {
+            new.output$pi.data <- do.call(rbind.data.frame, x$DatasetPIs)
+            rownames(new.output$pi.data) <- NULL
+          }
 
           sub.test <- try(do.call(rbind.data.frame, x$SubDates))
 
-          if(length(sub.test) > 0){
+          if (length(sub.test) > 0) {
               colnames(sub.test) <- c("submission.date",  "submission.type")
               sub.test$submission.date <- as.character(sub.test$submission.date)
               sub.test$submission.type <- as.character(sub.test$submission.type)
@@ -188,29 +208,38 @@ get_dataset.default <- function(x, datasettype, piid, altmin, altmax, loc, gpid,
 #'
 #' @param x An object of class \code{site}.
 #' @param ... objects passed from the generic.  Not used in the call.
-#' @importFrom RCurl getForm
-#' @importFrom RJSONIO fromJSON
+#' @importFrom jsonlite fromJSON
+#' @importFrom httr GET content
 #' @export
-get_dataset.site <- function(x, ...){
-
-  pull_site <- function(siteid){
+get_dataset.site <- function(x, ...) {
 
+  pull_site <- function(siteid) {
+    
     base.uri <- 'http://api.neotomadb.org/v1/data/datasets/?siteid='
-    aa <- try(fromJSON(paste0(base.uri, siteid), nullValue = NA))
+    
+    neotoma_content <- httr::content(httr::GET(paste0(base.uri, siteid)), as = "text")
+    if (identical(neotoma_content, "")) stop("")
+    aa <- jsonlite::fromJSON(neotoma_content, simplifyVector = FALSE)
 
-    if (aa[[1]] == 0){
+    if (aa[[1]] == 0) {
       stop(paste('Server returned an error message:\n', aa[[2]]), call. = FALSE)
     }
-    if (aa[[1]] == 1){
+    if (aa[[1]] == 1) {
       output <- aa[[2]]
+      # replace NULL values:
+      
     }
 
-    if(length(output) == 0){
+    if (length(output) == 0) {
       warning('The criteria used returned 0 sample sites. Returning NULL.')
       return(NULL)
     }
+    
     new.output <- lapply(output, function(x) {
       new.output <- list()
+      
+      x <- lapply(x, function(x) ifelse(x == "NULL" | class(x) == 'logical', NA, x))
+      
       new.output$site.data <- data.frame(site.id = x$Site$SiteID,
                                          site.name = x$Site$SiteName,
                                          long = mean(unlist(x$Site[c('LongitudeWest', 'LongitudeEast')]),
@@ -223,24 +252,33 @@ get_dataset.site <- function(x, ...){
                                          lat.acc = abs(x$Site$LatitudeNorth - x$Site$LatitudeSouth),
                                          row.names = x$Site$SiteName,
                                          stringsAsFactors = FALSE)
-      new.output$dataset.meta <- data.frame(dataset.id = x$DatasetID,
-                                       dataset.name = x$DatasetName,
-                                       collection.type = x$CollUnitType,
-                                       collection.handle = x$CollUnitHandle,
-                                       dataset.type = x$DatasetType,
-                                       stringsAsFactors = FALSE)
-      new.output$pi.data <- do.call(rbind.data.frame, x$DatasetPIs)
-      rownames(new.output$pi.data) <- NULL
-
+      
+      new.output$dataset.meta <- data.frame(dataset.id = ifelse(class(x$DatasetID) == 'logical',
+                                                                NA, x$DatasetID),
+                                            dataset.name = ifelse(class(x$DatasetName) == 'logical',
+                                                                  NA, x$DatasetName),
+                                            collection.type = ifelse(class(x$CollUnitType) == 'logical',
+                                                                     NA, x$CollUnitType),
+                                            collection.handle = ifelse(class(x$CollUnitHandle) == 'logical',
+                                                                       NA, x$CollUnitHandle),
+                                            dataset.type = ifelse(class(x$DatasetType) == 'logical',
+                                                                  NA, x$DatasetType),
+                                            stringsAsFactors = FALSE)
+      if (class(x$DatasetPIs) == 'logical') { 
+        new.output$pi.data <- NA
+      } else {
+        new.output$pi.data <- do.call(rbind.data.frame, x$DatasetPIs)
+        rownames(new.output$pi.data) <- NULL
+      }
+      
       sub.test <- try(do.call(rbind.data.frame, x$SubDates))
 
-      if(length(sub.test) > 0){
+      if (length(sub.test) > 0) {
         colnames(sub.test) <- c("SubmissionDate",  "SubmissionType")
       }
 
       new.output$submission <- sub.test
 
-
       new.output$access.date = Sys.time()
 
       class(new.output) <- c('dataset', 'list')
@@ -264,7 +302,7 @@ get_dataset.site <- function(x, ...){
 #' @param x An object of class \code{download}.
 #' @param ... objects passed from the generic.  Not used in the call.
 #' @export
-get_dataset.download <- function(x, ...){
+get_dataset.download <- function(x, ...) {
   # Just pull the dataset out of the download.
   output <- list(x$dataset)
 
@@ -282,10 +320,10 @@ get_dataset.download <- function(x, ...){
 #' @param x An object of class \code{download_list}.
 #' @param ... objects passed from the generic.  Not used in the call.
 #' @export
-get_dataset.download_list <- function(x, ...){
+get_dataset.download_list <- function(x, ...) {
 
   # Just pull the dataset out of the download and reassign classes:
-  output <- lapply(x, FUN=function(y){
+  output <- lapply(x, FUN=function(y) {
     dataset <- y$dataset
     class(dataset) <- c('dataset', 'list')
     dataset })
@@ -303,7 +341,7 @@ get_dataset.download_list <- function(x, ...){
 #' @param x An object of class \code{geochronologic}.
 #' @param ... objects passed from the generic.  Not used in the call.
 #' @export
-get_dataset.geochronologic <- function(x, ...){
+get_dataset.geochronologic <- function(x, ...) {
   x[[1]]
 }
 
@@ -313,7 +351,7 @@ get_dataset.geochronologic <- function(x, ...){
 #' @param x An object of class \code{geochronologic_list}.
 #' @param ... objects passed from the generic.  Not used in the call.
 #' @export
-get_dataset.geochronologic_list <- function(x, ...){
+get_dataset.geochronologic_list <- function(x, ...) {
   out <- lapply(x, function(y)y[[1]])
   class(out) <- c('dataset_list', 'list')
   out
