@@ -1,8 +1,9 @@
 #' @title Function to return full download records using \code{site}s, \code{dataset}s, or dataset IDs.
 #' @description Using the dataset ID, site object or dataset object, return all records associated with the data as a \code{download_list}.
 #'
-#' @importFrom RJSONIO fromJSON
-#' @importFrom reshape2 dcast
+#' @importFrom jsonlite fromJSON
+#' @importFrom httr content GET
+#' @importFrom stats na.omit
 #' @param x A single numeric dataset ID or a vector of numeric dataset IDs as returned by \code{get_datasets}, or a \code{site}, \code{dataset}, or \code{dataset_list}.
 #' @param verbose logical; should messages on API call be printed?
 #' @author Simon J. Goring \email{simon.j.goring@@gmail.com}
@@ -19,6 +20,7 @@
 #' @section Note:
 #' The function returns a warning in cases where single taxa are defined by multiple taphonomic characteristics, for example grains that are identified seperately as crumpled and torn in the same sample and sums these values within a sample.
 #' In the case that a geochronology dataset is passed to \code{get_download} the function returns a message and a NULL object (that is later excized).  Use \code{get_geochron} for these objects.
+#' The chronologies can be augmented using the function \code{get_chroncontrol}, where the individual chronology objects in \code{chronologies} will consist of a table equivalent to \code{sample.meta} and a \code{chroncontrol} object.
 #'
 #' @examples \dontrun{
 #' #  Search for sites with "Pseudotsuga" pollen that are older than 8kyr BP and
@@ -34,19 +36,25 @@
 #'
 #' #  Extract the Pseudotsuga curves for the sites:
 #' get.curve <- function(x, taxa) {
+#'                if (taxa %in% colnames(x$counts)) {
+#'                  count <- x$counts[,taxa]/rowSums(x$counts, na.rm=TRUE)
+#'                } else {
+#'                  count <- rep(0, nrow(x$count))
+#'                }
 #'                data.frame(site = x$dataset$site.data$site.name,
 #'                age = x$sample.meta$age,
-#'                count = x$counts[,taxa]/rowSums(x$counts, na.rm=TRUE))
+#'                count = count)
 #'              }
 #'
 #' curves <- do.call(rbind.data.frame,
-#'                   lapply(compiled.sites, get.curve, taxa = 'Picea'))
+#'                   lapply(compiled.sites, get.curve, taxa = 'Larix/Pseudotsuga'))
 #'
 #' #  For illustration, remove the sites with no Pseudotsuga occurance:
 #' curves <- curves[curves$count > 0, ]
 #'
 #' smooth.curve <- predict(loess(sqrt(count)~age, data=curves),
 #'                         data.frame(age=seq(20000, 0, by = -100)))
+#'                         
 #' plot(sqrt(count) ~ age, data = curves,
 #'      ylab = '% Pseudotsuga/Larix', xlab='Calibrated Years BP', pch=19,
 #'      col=rgb(0.1, 0.1, 0.1, 0.1), xlim=c(0, 20000))
@@ -60,18 +68,19 @@
 #' API Reference:  http://api.neotomadb.org/doc/resources/contacts
 #' @keywords IO connection
 #' @export
-get_download <- function(x, verbose = TRUE){
+get_download <- function(x, verbose = TRUE) {
   UseMethod('get_download')
 }
 
 #' @title Function to return full download records using \code{numeric} dataset IDs.
 #' @description Using the dataset ID, return all records associated with the data as a \code{download_list}.
 #'
-#' @importFrom RJSONIO fromJSON
+#' @importFrom jsonlite fromJSON
+#' @importFrom httr content GET
 #' @param x A single numeric dataset ID or a vector of numeric dataset IDs as returned by \code{get_datasets}.
 #' @param verbose logical; should messages on API call be printed?
 #' @export
-get_download.default <- function(x, verbose = TRUE){
+get_download.default <- function(x, verbose = TRUE) {
 
   # Updated the processing here. There is no need to be fiddling with
   # call. Use missing() to check for presence of argument
@@ -84,12 +93,14 @@ get_download.default <- function(x, verbose = TRUE){
           stop('The dataset id (x) must be numeric.')
   }
 
-  get.sample <- function(x){
+  get.sample <- function(x) {
     # query Neotoma for data set
     base.uri <- 'http://api.neotomadb.org/v1/data/downloads'
     
-    aa <- try(fromJSON(paste0(base.uri, '/', x), nullValue = NA))
-
+    neotoma_content <- httr::content(httr::GET(paste0(base.uri, '/', x)), as = "text")
+    if (identical(neotoma_content, "")) stop("")
+    aa <- jsonlite::fromJSON(neotoma_content, simplifyVector = FALSE)
+    
     # Might as well check here for error and bail
     if (inherits(aa, "try-error"))
         return(aa)
@@ -101,8 +112,28 @@ get_download.default <- function(x, verbose = TRUE){
     }
 
     if (isTRUE(all.equal(aa[[1]], 1))) {
-        aa <- aa[[2]]
 
+      if (aa[[1]] == 1 & length(aa[[2]]) == 0) {
+        # Is this the best way to deal with it?
+        message(paste0("Dataset ID ", x, " has no associated record in Neotoma."))
+        return(NULL)
+      }
+        aa <- aa[[2]]
+        
+        rep_NULL <- function(x) { 
+          # small function to recursively fill all NULL values with NAs.
+          if (is.null(x)) {NA}
+          else{
+            if (class(x) == 'list') {
+              lapply(x, rep_NULL)
+            } else {
+              return(x)
+            }
+          }
+        }
+        
+        aa <- lapply(aa, function(x)rep_NULL(x))
+        
         if (verbose) {
             message(strwrap(paste0("API call was successful. ",
                                    "Returned record for ",
@@ -144,28 +175,25 @@ get_download.default <- function(x, verbose = TRUE){
             submission = data.frame(submission.date = strptime(aa1$NeotomaLastSub,
                                                                '%m/%d/%Y'),
                                     submission.type = 'Last submission to Neotoma',
-                                    stringsAsFactors=FALSE),
+                                    stringsAsFactors = FALSE),
             access.date = Sys.time())
 
+          # Assign classes
+          
           class(dataset) <- c('dataset', 'list')
           class(dataset$site) <- c('site', 'data.frame')
           
           #  Geochronological datasets behave differently than any other dataset.
           #  It's frustrating.  This is the only way we can figure it out in a
           #  general way.
-          if(dataset$dataset.meta$dataset.type == 'geochronologic'){
+          if (dataset$dataset.meta$dataset.type == 'geochronologic') {
             
             message(paste0('The dataset ID ', dataset$dataset.meta$dataset.id,
                            ' is associated with a geochronology object, not count data.'))
             return(NULL)
-            #geochron <- get_geochron(dataset$dataset.meta$dataset.id)
+
+          } else {
             
-            #aa <- list(dataset, geochron = geochron)
-            #class(aa) <- 'geochron'
-            #return(aa)
-          }
-            
-          if(!dataset$dataset.meta$dataset.type == 'geochronologic'){
             # copy to make indexing below easier?
             samples <- aa1$Samples
 
@@ -188,44 +216,179 @@ get_download.default <- function(x, verbose = TRUE){
 
             # First, get all unique chronology names.
             # Some cores don't have age models, so we use a try function.
-            chrons <- try(unique(as.vector(unlist(sapply(samples,
-                                                         function(x)sapply(x$SampleAges, function(x)x$ChronologyName))))), silent = TRUE)
+            
+            # There is an issue in about 31 records in the US database (where I tested)
+            # where there are samples without named sample ages, but not a second chronology.
+            # The issue is that there are undated samples within a record that has dated records.
+            # So some samples are NAPD1 and then the undated samples, deeper in the core are just 'NA'
+            # This means that NA is just a continuous flow from NAPD1.
+            
+            # In records with two chronologies we get an array with two rows.
+            # If there are unidentified samples within the chronology they aren't named.
+            # This becomes problematic when we have `n` chronologies, plus samples without
+            # named chronologies.
+            
+            chron_list <- lapply(samples, '[[', 'SampleAges')
+            
+            chron_names <- unique(unlist(sapply(chron_list, function(x)unique(sapply(x, '[[', 'ChronologyName')))))
+            chron_lengths <- sapply(chron_list, length)
+            # When there is an NA aged sample in a record it fucks everything up
+            # because it's not inherently a part of a chronology.
+            
+            if (any(is.na(chron_names)) & !all(diff(chron_lengths) == 0)) {
+              # This implies that many records have multiple chronology
+              # coverage, but that some don't & have NA coverage.
+              # In that case we want to duplicate the record, and then
+              # assign names to the chronologies.
+              reassign <- function(x, chron_names) {
+                if (length(x) == length(stats::na.omit(chron_names))) {
+                  return(x)
+                } else {
+                  x <- rep(x, max(chron_lengths))
+                  return(lapply(1:max(chron_lengths), 
+                          function(y) {x[[y]]$ChronologyName <- chron_names[y]; x[[y]]}))
+                }
+              }
+            } else if (all(is.na(chron_names))) {
+              chron_names <- 'No chronology'
+              chron_list <- lapply(chron_list, function(x) {x[[1]]$ChronologyName <- chron_names; x})
+            } else if (any(is.na(chron_names)) & all(diff(chron_lengths) == 0)) {
+                # This implies that many records have multiple chronology
+                # coverage, but that some don't & have NA coverage.
+                # In that case we want to duplicate the record, and then
+                # assign names to the chronologies.
+                reassign <- function(x, chron_names) {
+                  if (length(x) == length(na.omit(chron_names)) & !is.na(x[[1]]$ChronologyName)) {
+                    return(x)
+                  } else {
+                    x <- rep(x, max(chron_lengths))
+                    return(lapply(1:max(chron_lengths), 
+                                  function(y) {x[[y]]$ChronologyName <- chron_names[y]; x[[y]]}))
+                  }
+                }
+                chron_list <- lapply(chron_list, reassign, chron_names = chron_names)   
+            }
+            
+            chron_vectors <- sapply(chron_list, function(x)unique(sapply(x, '[[', 'ChronologyName')))
+
+            if (is.list(chron_vectors)) {
+              #  If it's a list then there are levels without common chronologies
+              #  in practice this is more likely for faunal/macrofossil remains.
+              unique_vectors <- stats::na.omit(unique(unlist(chron_vectors)))
+              
+              # Now, we inject an empty age for the missing chronology:
+              for (i in unique_vectors) {
+                for (j in 1:length(chron_list)) {
+                  if (!i %in% sapply(chron_list[[j]], '[[', 'ChronologyName')) {
+                    if (any(is.na(sapply(chron_list[[j]], '[[', 'ChronologyName')))) {
+                      # Clear any empty records:
+                      chron_list[[j]][[which(is.na(sapply(chron_list[[j]], '[[', 'ChronologyName')))]] <- NULL
+                    }
+                    add_length <- length(chron_list[[j]]) + 1
+                    
+                    chron_list[[j]][[add_length]] <- list(AgeOlder = NA,
+                                                          Age = NA,
+                                                          AgeYounger = NA,
+                                                          ChronologyName = i,
+                                                          AgeType = NA,
+                                                          ChronologyID = NA)
+                    
+                  }
+                }
+              }
+              
+              # Now we just need to re-sort the lists:
+              chron_list <- lapply(chron_list, function(x){
+                lapply(match(sapply(x, '[[', 'ChronologyName'), unique_vectors), function(y)x[[y]])
+              })
+              
+              chron_vectors <- sapply(chron_list, function(x)unique(sapply(x, '[[', 'ChronologyName')))
+              
+            }
+            # This fills in the end of a set of sample ages if there are NAs in a series,
+            # but with the fix above it shouldn't be necessary.
+            if (!is.null(nrow(chron_vectors))) {
+              # There's a problem here that the things aren't in the right order.
+              
+              chron_vectors <- t(apply(chron_vectors, 1, function(x) {
+                                        if (any(is.na(x)) & !all(is.na(x))) {
+                                          x[is.na(x)] <- unique(x[!is.na(x)])
+                                        }
+                                    x } ))
+            } else {
+              chron_vectors[is.na(chron_vectors)] <- unique(unlist(chron_vectors[!is.na(chron_vectors)]))
+            }
+            
+            chrons <- try(unique(as.vector(unlist(chron_vectors))), silent = TRUE)
 
             base.frame <- as.data.frame(matrix(ncol = 7,
                                                nrow = nrow(sample.meta)))
+            
             colnames(base.frame) <- c('age.older', 'age',
                                       'age.younger', 'chronology.name',
                                       'age.type', 'chronology.id', 'dataset.id')
 
-            if (!class(chrons) == 'try-error'){
+            if (!class(chrons) == 'try-error') {
+              # Now we create the chronologies, so long as samples have assigned "SampleAges"
+              # If they don't, then we stick in the empty `base.frame` and assign it a name "1"
               # Create the list:
-              chron.list <- list()
-              for (i in 1:length(chrons)) chron.list[[i]] <- base.frame
-              names(chron.list) <- chrons
+              chron.list <- lapply(1:length(chrons), function(x) base.frame)
+              
+              if (!(is.null(chrons) | any(is.na(chrons))) & length(chrons) > 0) {
+                # If there is a chronology and there are no NAs in it:
+                names(chron.list) <- chrons
 
-              for (i in 1:length(samples)){
-                for (j in 1:length(samples[[i]]$SampleAges)){
-                  # Some of the new datasets are passing data without any chronology information.
-                  if(!is.na(samples[[i]]$SampleAges[[j]]['ChronologyName'])) {
+                for (i in 1:length(samples)) {
                   
-                    chron.list[[ samples[[i]]$SampleAges[[j]]$ChronologyName ]][i, ] <-
-                      data.frame(samples[[i]]$SampleAges[[j]],
-                                 stringsAsFactors = FALSE)
+                  if (length(chron_list[[i]]) < length(chrons) & length(chron_list[[i]]) == 1) {
+                    #  If there's a sample with nothing in it then it gets only a single
+                    # chron_list object, which makes the `j` loop choke.
+                    #  This current implementation only accounts for a completely
+                    # undated sample within the core, but not if one model spans a different length
+                    # of the core.
+                    
+                    for (j in 1:length(chrons)) {
+                      # Fix the `j` placeholder at 1
+                      chron.list[[j]][i, ] <- data.frame(chron_list[[i]][[1]],
+                                                         stringsAsFactors = FALSE)
+                      chron.list[[j]]$dataset.id <- dataset$dataset.meta$dataset.id
+                    }
+                    
+                  } else if (!is.null(chrons) & any(is.na(chrons)) & length(chrons) > 0) {
+                    # There is a chronology, but one of levels has an NA:
+                    for (i in 1:length(chron_list)) {
+                      chron_list
+                    }
+                    
+                  } else {
+                    for (j in 1:length(chrons)) {
+                      # Some of the new datasets are passing data without any chronology information.
+                      # Here we're filling in the dataset metadata
+                      
+                      chron.list[[j]][i, ] <- data.frame(chron_list[[i]][[j]],
+                                                         stringsAsFactors = FALSE)
+                      chron.list[[j]]$dataset.id <- dataset$dataset.meta$dataset.id
+                    }
                   }
-                  chron.list[[samples[[i]]$SampleAges[[j]]$ChronologyName]]$dataset.id <- dataset$dataset.meta$dataset.id
                 }
+              } else {
+                chron.list[[ 1 ]][1, ] <-
+                  data.frame(samples[[1]]$SampleAges[[1]],
+                             stringsAsFactors = FALSE)
+                chron.list[[1]]$dataset.id <- dataset$dataset.meta$dataset.id
               }
               
-              default_chron <- which(sapply(chron.list, function(x)x$chronology.id[1])==aa1$DefChronologyID)
+              default_chron <- which(sapply(chron.list, function(x)x$chronology.id[1]) == aa1$DefChronologyID)
+              if (length(default_chron) > 1) { default_chron <- default_chron[1] }
               
-              if(length(default_chron)==0){
+              if (length(default_chron) == 0) {
                 warning(paste0('This dataset has no defined default chronology.  Please use caution.\n',
-                        'Using the first chronology, ', names(chron.list)[1],' as the default.'))
+                        'Using the first chronology, ', names(chron.list)[1],' as the default.'),
+                        immediate. = TRUE)
                 default_chron <- 1
               }
               
-            }
-            if (class(chrons) == 'try-error'){
+            } else {
               chron.list <- list(base.frame)
               default_chron <- 1
             }
@@ -254,11 +417,19 @@ get_download.default <- function(x, verbose = TRUE){
             # list of data frames
             sample.data <- lapply(lapply(samples, `[[`, "SampleData"),
                                   function(x) do.call(rbind.data.frame, x))
+            
+            if (length(unlist(sample.data)) == 0) {
+              warning('This record contains no count data.  Returning a NULL record.')
+              return(NULL)
+            }
+            
             # 2) How many counts/species in each data frame?
             nsamp <- sapply(sample.data, nrow)
+            
             # 3) bind each data frame - result is a data frame in long format
             sample.data <- do.call(rbind, sample.data)
-            # 4) add a Sample column that is the ID from smaple.meta
+            
+            # 4) add a Sample column that is the ID from sample.meta
             sample.data$sample.id <- rep(sample.meta$sample.id, times = nsamp)
 
             # We're going to isolate the count data and clean it up by
@@ -270,116 +441,80 @@ get_download.default <- function(x, verbose = TRUE){
                                        'taxon.group', 'value',
                                        'ecological.group', 'sample.id')
             
-            take <- !(sample.data$taxon.group == "Laboratory analyses" |
-                        sample.data$taxon.group == "Charcoal")
-
-            count.data <- sample.data[take, ]
-
-            # Ensure duplicate taxa are renamed
-            # (if variable context is different)
-            count.data$taxon.name <- as.character(count.data$taxon.name)
-            var.context <- !is.na(count.data$variable.context)
-            count.data$taxon.name[var.context] <- paste(count.data$taxon.name,
-                                                       count.data$variable.context,
-                                                       sep = '.')[var.context]
-
-            # data frame of unique taxon info.  This gets included in the
-            # final dataset output by the function.
-            taxon.list <- sample.data[!duplicated(sample.data$taxon.name),
-                                      1:5]
-
-            # Some taxa/objects get duplicated because different identifiers
-            # for taphonomic modification get excluded in the API table.
-            # Because the data is excluded we can't be sure that the
-            # modifications map exactly from sample to sample, so here we
-            # just sum all duplicated taxa and throw a warning to the user:
-            mod.dups <- duplicated(count.data[, c('taxon.name', 'sample.id')])
-
-            if (sum(mod.dups) > 0){
-              tax.dups <- unique(count.data$taxon.name[duplicated(count.data[, c('taxon.name', 'sample.id')])])
-              if (length(tax.dups) == 1){
-                message <- paste0('\nModifiers seem absent from the taxon ',
-                                  tax.dups,
-                                  '. \nget_download will sum at depths ',
-                                  'with multiple entries to resolve the problem.')
-              }
-              if (length(tax.dups) > 1){
-                tax.dups.col <- paste(tax.dups, collapse = ', ')
-                message <- paste0('\nModifiers seem absent from the taxons ',
-                                 tax.dups.col,
-                                 '. \nget_download will sum at depths with ',
-                                 'multiple entries to resolve the problem.')
-              }
-              warning (immediate. = TRUE, message, call. = FALSE)
-            }
-
-            # reshape long sample.data into a sample by taxon data frame
-            # take here *only* counts - but needs work FIXME
+            # get the table:
+            cast_table <- reshape2::dcast(sample.data, 
+                                taxon.name + variable.units + variable.element +
+                                  variable.context + taxon.group +
+                                  ecological.group ~ sample.id, value.var = "value")
             
-            if(nrow(count.data) > 0){
-              counts <- dcast(count.data,
-                              formula = sample.id ~ taxon.name,
-                              value.var = "value",
-                              fun.aggregate = sum, na.rm = TRUE)
-  
-              # add Sample col as the rownames
-              rownames(counts) <- counts$sample.id
-              ## remove the Sample col, but robustly
-              counts <- counts[, -which(names(counts) == "sample.id"), drop = F]
-  
-              # It is possible that some depths have no count data,
-              # but that they were sampled. This will be
-              # reflected as a row with '0' counts for all taxa.
-              if (any(!sample.meta$sample.id %in% rownames(counts))){
-                no.missing <- sum(!sample.meta$sample.id %in% rownames(counts))
-  
-                for (i in 1:no.missing){
-                  counts <- rbind(counts, rep(NA, ncol(counts)))
+            taxon.list <- cast_table[ ,1:6]
+            
+            # Now we check for any duplicated names and give them an alias
+            # that binds the name with the variable units.
+            if (any(duplicated(cast_table$taxon.name))) {
+              
+              which_dup <- as.character(taxon.list$taxon.name[duplicated(taxon.list$taxon.name)])
+              
+              # A variable may be duplicated because the same variable was mapped in
+              # different units, or because it comes from different contexts.
+              # So we want to replace with the appropriate name:
+              
+              taxon.list$alias <- as.character(taxon.list$taxon.name)
+              
+              for (i in which_dup) {
+                # Choose the column to resolve the problem.
+                dup_rows <- data.frame(taxon.list[taxon.list$taxon.name %in% i,],
+                                       stringsAsFactors = FALSE)
+                
+                # The pick order should be:
+                # 1. variable.context
+                # 2. variable.element - it's clear
+                # 3. variable.unit - not sure how often the same element is expressed in different units.
+                # 4. context|element
+                dup_rows <- dup_rows[,c('variable.context','variable.element',
+                                        'variable.units')]
+                
+                # Remove any columns where all elements are NA:
+                dup_rows <- dup_rows[, apply(dup_rows, 2, function(x) { !all(is.na(x)) } )]
+                
+                #  To resolve the duplication issue we want to find the shortest combination of columns for
+                #  which the sum of duplicates is zero:
+                if (is.null(nrow(dup_rows))) {
+                  # In some cases there are lots of NAs. . . 
+                  taxon.list$alias[taxon.list$taxon.name %in% i] <-  paste0(taxon.list$alias[taxon.list$taxon.name %in% i], '|',
+                                                                            as.character(dup_rows))
+                } else {
+                  taxon.list$alias[taxon.list$taxon.name %in% i] <- sapply(1:nrow(dup_rows),
+                                                                           function(x) {
+                                                                             paste0(taxon.list$alias[taxon.list$taxon.name %in% i][x], '|',
+                                                                                             paste0(as.character(t(dup_rows)[,x]), collapse = '|')) })
                 }
-                rownames(counts)[(nrow(counts) + 1 - no.missing):nrow(counts)] <- sample.meta$sample.id[!sample.meta$sample.id %in% rownames(counts)]
-  
-                counts <- counts[as.character(sample.meta$sample.id), ]
               }
               
+              message <- paste0('\nThere were multiple entries for ',
+                                unique(which_dup),
+                                sapply(unique(which_dup), function(x)ifelse(length(grep("\\.$", 'abcd', perl = TRUE)) == 1, '.', '')),
+                                ' \nget_download has mapped aliases for the taxa in the taxon.list.')
+              warning(immediate. = TRUE, message, call. = FALSE)
+            }  
+
+            # This pulls out charcoal & lab data to stick in a different data.frame.
+            take <- !(taxon.list$taxon.group == "Laboratory analyses" |
+                        taxon.list$taxon.group == "Charcoal")
+
+            count.data <- t(cast_table[take, 7:ncol(cast_table)])
+            if ('alias' %in% colnames(taxon.list)) {
+              colnames(count.data) <- taxon.list$alias[take]
             } else {
-              counts <- NULL
+              colnames(count.data) <- taxon.list$taxon.name[take]
             }
-  
+
             # Pull out the lab data and treat it in
             # the same way as the previous:
-            take <- sample.data$taxon.group == "Laboratory analyses" |
-              sample.data$taxon.group == "Charcoal"
-
-            lab.data <- sample.data[take, ]
-
-            if (nrow(lab.data) > 0) {
-                lab.data$lab.name.units <- paste0(lab.data$taxon.name, " (",
-                                                lab.data$variable.element, ": ",
-                                                lab.data$variable.units, ")")
-
-                mod.dups <- duplicated(lab.data[, c(1, 7)])
-
-                if (sum(mod.dups) > 0){
-                  lab.dups <- unique(lab.data$taxon.name[duplicated(lab.data[, c(1, 7)])])
-                  if (length(lab.dups) == 1){
-                    message <- paste0('\nModifiers are absent from the lab object ',
-                                      lab.dups,
-                                      '. \nget_download will use unique',
-                                      'identifiers to resolve the problem.')
-                  }
-                  if (length(lab.dups) > 1){
-                    lab.dups.col <- paste(lab.dups, collapse = ', ')
-                    message <- paste0('\nModifiers are absent from the lab objects ',
-                                      lab.dups.col,
-                                      '. \nget_download will use unique',
-                                      'identifiers to resolve the problem.')
-                  }
-                  warning (immediate. = TRUE, message, call. = FALSE)
-                }
-
-                lab.data <- dcast(lab.data,
-                                  formula = sample.id ~ lab.name.units,
-                                  value.var = "value")
+            
+            if (sum(take) > 0) {
+              lab.data <- t(cast_table[!take, 7:ncol(cast_table)])
+              colnames(lab.data) <- taxon.list$alias[!take]
             } else {
                 lab.data <- NULL
             }
@@ -388,7 +523,7 @@ get_download.default <- function(x, verbose = TRUE){
             aa <- list(dataset = dataset,
                        sample.meta = sample.meta,
                        taxon.list = taxon.list,
-                       counts = counts,
+                       counts = count.data,
                        lab.data = lab.data,
                        chronologies = chron.list)
           
@@ -410,8 +545,8 @@ get_download.default <- function(x, verbose = TRUE){
 
   drop.any <- sapply(aa,is.null)
 
-  if(any(drop.any > 0)){
-    if(all(drop.any > 0)){
+  if (any(drop.any > 0)) {
+    if (all(drop.any > 0)) {
       stop('All datasets return non-download objects\n')
     } else {
       warning('Some datasetids returned empty downloads, be aware that length(datasetid) is longer than the download_list.\n')
@@ -430,11 +565,10 @@ get_download.default <- function(x, verbose = TRUE){
 #' @title Function to return full download records using a \code{dataset}.
 #' @description Using a \code{dataset}, return all records associated with the data as a \code{download_list}.
 #'
-#' @importFrom RJSONIO fromJSON
 #' @param x An object of class \code{dataset}.
 #' @param verbose logical; should messages on API call be printed?
 #' @export
-get_download.dataset <- function(x, verbose = TRUE){
+get_download.dataset <- function(x, verbose = TRUE) {
 
   # Updated the processing here. There is no need to be fiddling with
   # call. Use missing() to check for presence of argument
@@ -442,7 +576,7 @@ get_download.dataset <- function(x, verbose = TRUE){
   
   datasetid <- x$dataset.meta$dataset.id
   
-  if(!x$dataset.meta$dataset.type %in% 'geochronologic'){
+  if (!x$dataset.meta$dataset.type %in% 'geochronologic') {
     aa <- get_download(datasetid, verbose = verbose)
   } else {
     cat('Dataset is a geochronological data object.  Defaulting to get_geochron.\n')
@@ -457,11 +591,10 @@ get_download.dataset <- function(x, verbose = TRUE){
 #' @title Function to return full download records using a \code{dataset_list}.
 #' @description Using a \code{dataset_list}, return all records associated with the data as a \code{download_list}.
 #'
-#' @importFrom RJSONIO fromJSON
 #' @param x An object of class \code{dataset_list}.
 #' @param verbose logical; should messages on API call be printed?
 #' @export
-get_download.dataset_list <- function(x, verbose = TRUE){
+get_download.dataset_list <- function(x, verbose = TRUE) {
   
   # Updated the processing here. There is no need to be fiddling with
   # call. Use missing() to check for presence of argument
@@ -477,11 +610,10 @@ get_download.dataset_list <- function(x, verbose = TRUE){
 #' @title Function to return full download records using a \code{site}.
 #' @description Using a \code{site}, return all records associated with the data as a \code{download_list}.
 #'
-#' @importFrom RJSONIO fromJSON
 #' @param x An object of class \code{site}.
 #' @param verbose logical; should messages on API call be printed?
 #' @export
-get_download.site <- function(x, verbose = TRUE){
+get_download.site <- function(x, verbose = TRUE) {
   
   message('Fetching datasets for the site(s)')
   dataset <- get_dataset(x)
