@@ -41,9 +41,15 @@ write_agefile <- function(download, chronology = 1, path,
     
     if('download' %in% class(download[[1]])){download <- download[[1]]}
     
-    chron.controls <- get_chroncontrol(download,
-                                       verbose = FALSE)
-  
+    if(class(download$chronologies[[chronology]]) == 'list'){
+      # This is new.  We can push chroncontrols into the download object:
+      chron.controls <- download$chronologies[[chronology]]$chroncontrol
+    } else{
+      # Otherwise, we get them from the download object.
+      chron.controls <- get_chroncontrol(download,
+                                         verbose = FALSE)
+    }
+    
     if (nrow(chron.controls$chron.control) < 2){
       stop('Chronology must have more than a single date for proper analysis.')
     }
